<?php
/*
Plugin Name: Scissors
Plugin URI: http://stephanreiter.info/2008/10/wordpress-plugin-scissors/
Description: Scissors enhances WordPress' handling of images by introducing cropping, resizing, rotating, and watermarking functionality.
Version: 1.0.2
Author: Stephan Reiter
Author URI: http://stephanreiter.info/

Copyright (C) 2008  Stephan Reiter <stephan.reiter@gmail.com>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

function scissors_set_memory_limit()
{
	@ini_set('memory_limit', '256M');
	//@ini_set('memory_limit', WP_MEMORY_LIMIT);
}

$scissors_dirname = plugin_basename(dirname(__FILE__));
$scissors_locale_dir = PLUGINDIR . '/' . $scissors_dirname . '/languages';
load_plugin_textdomain('scissors', $scissors_locale_dir);

function scissors_create_image($width, $height)
{
	if($width <= 0 || $height <= 0)
		return FALSE;

	$image = imagecreatetruecolor($width, $height);
	if(is_resource($image))
	{
		// set default alpha-blending mode
		imagealphablending($image, false);
		imagesavealpha($image, true);
	}
	return $image;
}

function scissors_guess_mime_type($filename)
{
	$info = pathinfo($filename);
	switch($info['extension'])
	{
	case 'jpeg': case 'jpg': return 'image/jpeg';
	case 'png': return 'image/png';
	case 'gif': return 'image/gif';
	default: return '';
	}
}

function scissors_load_image($filename, $mime_type = '')
{
	if(!file_exists($filename))
		return FALSE;

	if($mime_type == '')
		$mime_type = scissors_guess_mime_type($filename);

	switch($mime_type)
	{
	case 'image/jpeg': $image = imagecreatefromjpeg($filename); break;
	case 'image/png': $image = imagecreatefrompng($filename); break;
	case 'image/gif': $image = imagecreatefromgif($filename); break;
	default: $image = FALSE; break;
	}

	if(is_resource($image))
	{
		// set default alpha-blending mode
		imagealphablending($image, false);
		imagesavealpha($image, true);
	}

	return $image;
}

function scissors_save_image($image, $filename, $mime_type = '')
{
	if($mime_type == '')
		$mime_type = scissors_guess_mime_type($filename);

	// set default alpha-blending mode
	imagealphablending($image, false);
	imagesavealpha($image, true);

	switch($mime_type)
	{
	case 'image/jpeg': return imagejpeg($image, $filename, 90);
	case 'image/png': return imagepng($image, $filename);
	case 'image/gif': return imagegif($image, $filename);
	default: return FALSE;
	}
}

// Automatic resizing functionality -------------------------------------------

function scissors_get_fullpath_from_metadata($metadata)
{
	$upload_path = trim(get_option('upload_path'));
	if(empty($upload_path)) $upload_path = WP_CONTENT_DIR . '/uploads';
	$upload_path = path_join(ABSPATH, $upload_path);
	$filename = path_join($upload_path, $metadata['file']);
	return file_exists($filename) ? $filename : FALSE;
}

function scissors_resize_full($metadata)
{
	$dstW = $srcW = intval($metadata['width']);
	$dstH = $srcH = intval($metadata['height']);
	if($srcW <= 0 || $srcH <= 0)
		return $metadata; // image dimensions are fishy ...

	// calculate scaled image dimensions
	$maxW = intval(get_option('full_size_w'));
	$maxH = intval(get_option('full_size_h'));
	if($maxW > 0 && $dstW > $maxW)
	{
		$dstH = $dstH * ($maxW / $dstW);
		$dstW = $maxW;
	}
	if($maxH > 0 && $dstH > $maxH)
	{
		$dstW = $dstW * ($maxH / $dstH);
		$dstH = $maxH;
	}
	if($dstW == $srcW && $dstH == $srcH)
		return $metadata; // no need to resize the image

	$filename = scissors_get_fullpath_from_metadata($metadata);
	if(!$filename) return $metadata; // failed to locate the file
	
	scissors_set_memory_limit();

	// load the source image, resize it, and overwrite the original file and update the metadata
	$src = scissors_load_image($filename);
	if(is_resource($src))
	{
		$dst = scissors_create_image($dstW, $dstH);
		if(is_resource($dst))
		{
			if(imagecopyresampled($dst, $src, 0, 0, 0, 0, $dstW, $dstH, $srcW, $srcH))
			{
				if(scissors_save_image($dst, $filename))
				{
					$metadata['width'] = $dstW; $metadata['height'] = $dstH;
					list($uwidth, $uheight) = wp_shrink_dimensions($metadata['width'], $metadata['height']);
					$metadata['hwstring_small'] = "height='$uheight' width='$uwidth'";
				}
			}
			imagedestroy($dst);
		}
		imagedestroy($src);
	}

	return $metadata;	
}

add_filter('wp_generate_attachment_metadata', 'scissors_resize_full');

function scissors_fullsize_fields()
{
?><fieldset><legend class="hidden"><?php _e('Full size', 'scissors') ?></legend>
<label for="full_size_w"><?php _e('Max Width'); ?></label>
<input name="full_size_w" type="text" id="full_size_w" value="<?php form_option('full_size_w'); ?>" class="small-text" />
<label for="full_size_h"><?php _e('Max Height'); ?></label>
<input name="full_size_h" type="text" id="full_size_h" value="<?php form_option('full_size_h'); ?>" class="small-text" />
<span class="setting-description"><?php _e('Specify 0 to disable resizing in a particular dimension.', 'scissors') ?></span>
</fieldset><?php
}

function scissors_whitelist_options($options)
{
	if(isset($options['media']))
	{
		$options['media'][] = 'full_size_w';
		$options['media'][] = 'full_size_h';
	}
	return $options;
}

if(strstr($_SERVER['REQUEST_URI'], 'options'))
{
	add_option('full_size_w', '0');
	add_option('full_size_h', '0');
	add_filter('whitelist_options', 'scissors_whitelist_options');
	add_filter('sanitize_option_full_size_w', 'intval');
	add_filter('sanitize_option_full_size_h', 'intval');
	$wp_settings_fields['media']['default'][] = array('title' => __('Full size', 'scissors'), 'callback' => 'scissors_fullsize_fields');
}

// Watermarking support -------------------------------------------------------

function scissors_get_watermark_configuration()
{
	$filename = path_join(ABSPATH, get_option('scissors_watermark_path'));
	list($ww, $wh) = file_exists($filename) ? getimagesize($filename) : array(0, 0);
	return array('ver' => 1, // NOTE: increment this if features are added and add appropriate checks to load_configuration
		'ww' => $ww, 'wh' => $wh,
		'doscale' => get_option('scissors_watermark_size'),
		'scale' => get_option('scissors_watermark_size_relative'),
		'horz' => get_option('scissors_watermark_halign'),
		'vert' => get_option('scissors_watermark_valign'));
}

function scissors_get_relative_filepath($filename)
{
	$upload_path = trim(get_option('upload_path'));
	if(empty($upload_path)) $upload_path = WP_CONTENT_DIR . '/uploads';
	$upload_path = path_join(ABSPATH, $upload_path);
	return str_replace($upload_path . '/', '', $filename);
}

function scissors_get_absolute_filepath($filename)
{
	$upload_path = trim(get_option('upload_path'));
	if(empty($upload_path)) $upload_path = WP_CONTENT_DIR . '/uploads';
	$upload_path = path_join(ABSPATH, $upload_path);
	return path_join($upload_path, $filename);
}

function scissors_get_postid_from_fullfilename($filename)
{
	global $wpdb;
	$relfile = scissors_get_relative_filepath($filename);
	$post = $wpdb->get_row($wpdb->prepare("SELECT post_id FROM $wpdb->postmeta WHERE meta_value = %s LIMIT 1", $relfile));
	return ($post != null) ? $post->post_id : FALSE;
}

function scissors_save_watermark_configuration($filename, $rectfilename)
{
	$postId = scissors_get_postid_from_fullfilename($filename);
	if(!$postId) return FALSE;

	$config = scissors_get_watermark_configuration();
	$config['rectfilename'] = addslashes(scissors_get_relative_filepath($rectfilename));

	return add_post_meta($postId, "_scissors_watermarking", $config, true) or
		update_post_meta($postId, "_scissors_watermarking", $config);
}

function scissors_load_watermark_configuration($postId)
{
	$config = get_post_meta($postId, '_scissors_watermarking', true);
	if($config == '') return FALSE;
	$config['rectfilename'] = scissors_get_absolute_filepath($config['rectfilename']);
	return $config;
}

function scissors_set_watermarking_state($postId, $enabled)
{
	if($enabled)
		return delete_post_meta($postId, '_scissors_watermarking_state');
	else
	{
		return add_post_meta($postId, '_scissors_watermarking_state', 'disabled', true) or
			update_post_meta($postId, '_scissors_watermarking_state', 'disabled');
	}
}

function scissors_is_watermarking_enabled($postId = '')
{
	if(get_option('scissors_watermark_enabled') != '1')
		return FALSE;

	$filename = path_join(ABSPATH, get_option('scissors_watermark_path'));
	list($ww, $wh) = file_exists($filename) ? getimagesize($filename) : array(0, 0);
	if($ww <= 0 || $wh <= 0)
		return FALSE;

	if($postId != '')
	{
		$state = get_post_meta($postId, '_scissors_watermarking_state', true);
		return $state != 'disabled';

		// check if watermarking has been enabled for a particular image
		// return ($postId != '') ? (scissors_load_watermark_configuration($postId) != FALSE) : TRUE;
	}

	return TRUE;
}

function scissors_find_rect_filename($filename)
{
	$rectfilename = FALSE;
	if(file_exists($filename))
	{
		$path_parts = pathinfo($filename);
		$sansext = substr($path_parts['basename'], 0, -(strlen($path_parts['extension']) + 1)); // +1 for the dot
		$pattern = '!' . preg_quote($sansext) . '-[0-9]+-rect' . preg_quote('.' . $path_parts['extension']) . '$!';
		$dir = opendir($path_parts['dirname']);
		while($file = readdir($dir))
		{
			if(preg_match($pattern, $file) == 1)
			{
				$rectfilename = $path_parts['dirname'] . '/' . $file;
				break;
			}					
		}
		closedir($dir);
	}
	return $rectfilename;
}

function scissors_delete_watermark_meta($filename)
{
	$postId = scissors_get_postid_from_fullfilename($filename);
	$config = $postId ? get_post_meta($postId, '_scissors_watermarking', true) : '';
	if($config != '') @unlink(scissors_get_absolute_filepath($config['rectfilename']));
	if($postId) delete_post_meta($postId, '_scissors_watermarking');

	if($config == '')
	{
		// try to find the file manually ... it seems that the
		// metadata in the db is already gone when delete_file is invoked
		$rectfilename = scissors_find_rect_filename($filename);
		if($rectfilename) @unlink($rectfilename);
	}

	return $filename; // needs this to support WP filter chain
}

function scissors_place_watermark($image, $config = '')
{
	if($config == '')
		$config = scissors_get_watermark_configuration();

	$iw = imagesx($image); $ih = imagesy($image);
	$ww = $config['ww']; $wh = $config['wh'];

	// scale down the watermark to occupy the desired area if requested
	if($config['doscale'] == 1)
	{
		$desiredarea = $iw * $ih * ($config['scale'] / 100.0);
		// ($ww * $s) * ($wh * $s) = $desiredArea;
		$s = sqrt($desiredarea / ($ww * $wh));
		if($s < 1) { $ww = $ww * $s; $wh = $wh * $s; }
	}

	// make sure that the watermark fits onto the original image, scale it down if necessary
	if($ww > $iw) { $wh = $wh * ($iw / $ww); $ww = $iw; }
	if($wh > $ih) { $ww = $ww * ($ih / $wh); $wh = $ih; }

	switch($config['horz'])
	{
	default: case 0: $l = 0; break; // left
	case 1: $l = ($iw - $ww) / 2; break; // center
	case 2: $l = $iw - $ww; break; // right
	}

	switch($config['vert'])
	{
	default: case 0: $t = 0; break; // top
	case 1: $t = ($ih - $wh) / 2; break; // middle
	case 2: $t = $ih - $wh; break; // bottom
	}

	return array($l, $t, $ww, $wh);
}

function scissors_build_rect_filename($filename)
{
	$path_parts = pathinfo($filename);
	$sansext = substr($path_parts['basename'], 0, -(strlen($path_parts['extension']) + 1)); // +1 for the dot
	$randid = time(); // add a random number to make access to the rect more difficult (would allow watermark-removal)
	$path_parts['basename'] = "{$sansext}-{$randid}-rect." . $path_parts['extension'];
	return $path_parts["dirname"] . '/' . $path_parts['basename'];
}

// the following functions are referenced by the resampling and resizing/cropping functionality ---

function scissors_remove_watermark($image, $postId)
{
	$config = scissors_load_watermark_configuration($postId);
	if($config)
	{
		$rect = scissors_load_image($config['rectfilename']);
		if(is_resource($rect))
		{
			imagealphablending($image, false);
			imagesavealpha($image, true);
			list($left, $top, $width, $height) = scissors_place_watermark($image, $config);
			imagecopy($image, $rect, $left, $top, 0, 0, $width, $height);
			imagedestroy($rect);
		}
	}
}

function scissors_watermark_image($image, $postId)
{
	$result = FALSE;
	if(scissors_is_watermarking_enabled($postId))
	{
		$watermark = scissors_load_image(path_join(ABSPATH, get_option('scissors_watermark_path')));
		if(is_resource($watermark))
		{
			imagesavealpha($image, false);
			imagealphablending($image, true);
			list($l, $t, $ww, $wh) = scissors_place_watermark($image);
			$result = imagecopyresampled($image, $watermark, $l, $t, 0, 0, $ww, $wh, imagesx($watermark), imagesy($watermark));
			imagedestroy($watermark);
		}
	}
	return $result;
}

function scissors_watermark_file($filename, $postId)
{
	$result = FALSE;
	if(scissors_is_watermarking_enabled($postId))
	{
		$image = scissors_load_image($filename);
		if(is_resource($image))
		{
			if(scissors_watermark_image($image, $postId))
				$result = scissors_save_image($image, $filename);
			imagedestroy($image);
		}
	}
	return $result;
}

function scissors_rebuild_watermark_meta($image, $postId)
{
	$fullfilename = get_attached_file($postId);

	$done = FALSE;
	if(scissors_is_watermarking_enabled($postId))
	{
		// determine the place and the size of the watermark if applied to this image
		list($left, $top, $width, $height) = scissors_place_watermark($image);

		// back up the part of the image that will be overlaid with the watermark
		$rect = scissors_create_image($width, $height);
		if(is_resource($rect))
		{
			if(imagecopy($rect, $image, 0, 0, $left, $top, $width, $height))
			{
				$oldconfig = scissors_load_watermark_configuration($postId);
				$rectfilename = ($oldconfig == FALSE) ? scissors_build_rect_filename($fullfilename) : $oldconfig['rectfilename'];
				if(scissors_save_image($rect, $rectfilename))
					$done = scissors_save_watermark_configuration($fullfilename, $rectfilename);
			}
			imagedestroy($rect);
		}
	}

	if(!$done)
		scissors_delete_watermark_meta($fullfilename);						
}

// ------------------------------------------------------------------------------------------------

function scissors_get_postid_from_metadata($metadata)
{
	global $wpdb;
	$post = $wpdb->get_row($wpdb->prepare("SELECT post_id FROM $wpdb->postmeta WHERE meta_value = %s LIMIT 1", $metadata['file']));
	return ($post != null) ? $post->post_id : FALSE;
}

function scissors_apply_initial_watermarks($metadata)
{
	$postId = scissors_get_postid_from_metadata($metadata);
	if($postId && scissors_is_watermarking_enabled($postId))
	{
		// load the full size image
		$fullfilename = get_attached_file($postId);
		if($fullfilename && file_exists($fullfilename))
		{
			scissors_set_memory_limit();
			$image = scissors_load_image($fullfilename);
			if(is_resource($image))
			{
				scissors_rebuild_watermark_meta($image, $postId);

				// apply the watermark to the full size image
				if(scissors_watermark_image($image, $postId))
					scissors_save_image($image, $fullfilename);

				imagedestroy($image);
			}
		}

		// apply the watermark to the intermediate images
		if(count($metadata['sizes']) > 0)
		{
			foreach($metadata['sizes'] as $size => $data)
			{
				$filename = path_join(dirname($fullfilename), $data['file']);
				$image = scissors_load_image($filename);
				if(is_resource($image))
				{
					if(scissors_watermark_image($image, $postId))
						scissors_save_image($image, $filename);
					imagedestroy($image);
				}
			}
		}
	}
	return $metadata;
}

add_filter('wp_generate_attachment_metadata', 'scissors_apply_initial_watermarks');
add_filter('wp_delete_file', 'scissors_delete_watermark_meta');

function scissors_watermarking()
{
?><p><?php _e('The Scissors plugin supports automatic watermarking of images. Please configure this feature here ...', 'scissors') ?></p><?php
}

function scissors_watermarking_switch()
{
?><input name="scissors_watermark_enabled" type="checkbox" id="scissors_watermark_enabled" value="1" <?php checked('1', get_option('scissors_watermark_enabled')); ?>/>
<label for="scissors_watermark_enabled"><?php _e('Add watermarks to images that are uploaded from now on.', 'scissors') ?></label><?php
}

function scissors_watermarking_path()
{
?><fieldset><legend class="hidden"><?php _e('Image', 'scissors') ?></legend>
<input name="scissors_watermark_path" type="text" id="scissors_watermark_path" value="<?php form_option('scissors_watermark_path'); ?>" class="regular-text code" />
<input type="button" class="button" value="<?php _e('Select Image', 'scissors') ?>" onclick="tb_show('', 'media-upload.php?type=image&amp;TB_iframe=true');"/>
<br /><span class="setting-description"><?php _e('Click the button, pick an image in the opening dialog, and use "Insert into Post" to make the selection.', 'scissors') ?></span>
</fieldset><?php
}

function scissors_watermarking_size()
{
?><fieldset><legend class="hidden"><?php _e('Size', 'scissors') ?></legend>
<input id="scissors_watermark_size_absolute" type="radio" value="0" name="scissors_watermark_size" <?php checked('0', get_option('scissors_watermark_size')); ?>/>
<label class="align_watermark" for="scissors_watermark_size_absolute"><?php _e('Do not resize the watermark.', 'scissors'); ?></label><br />
<input id="scissors_watermark_size_relative" type="radio" value="1" name="scissors_watermark_size" <?php checked('1', get_option('scissors_watermark_size')); ?>/>
<?php
$field = '<input name="scissors_watermark_size_relative" type="text" value="' . attribute_escape(get_option('scissors_watermark_size_relative')) . '" class="small-text" />';
echo sprintf(__('Limit the watermark to %s percent of the destination image area.', 'scissors'), $field);
?>
</fieldset><?php
}

function scissors_watermarking_halign()
{
?><fieldset><legend class="hidden"><?php _e('Horizontal alignment', 'scissors') ?></legend>
<input id="scissors_watermark_halign_left" type="radio" value="0" name="scissors_watermark_halign" <?php checked('0', get_option('scissors_watermark_halign')); ?> />
<label class="align_watermark" for="scissors_watermark_halign_left"><?php _e('left'); ?></label>
<input id="scissors_watermark_halign_center" type="radio" value="1" name="scissors_watermark_halign" <?php checked('1', get_option('scissors_watermark_halign')); ?> />
<label class="align_watermark" for="scissors_watermark_halign_center"><?php _e('center'); ?></label>
<input id="scissors_watermark_halign_right" type="radio" value="2" name="scissors_watermark_halign" <?php checked('2', get_option('scissors_watermark_halign')); ?> />
<label class="align_watermark" for="scissors_watermark_halign_right"><?php _e('right'); ?></label>
</fieldset><?php
}

function scissors_watermarking_valign()
{
?><input id="scissors_watermark_valign_top" type="radio" value="0" name="scissors_watermark_valign" <?php checked('0', get_option('scissors_watermark_valign')); ?> />
<label class="align_watermark" for="scissors_watermark_valign_top"><?php _e('top'); ?></label>
<input id="scissors_watermark_valign_middle" type="radio" value="1" name="scissors_watermark_valign" <?php checked('1', get_option('scissors_watermark_valign')); ?> />
<label class="align_watermark" for="scissors_watermark_valign_middle"><?php _e('middle'); ?></label>
<input id="scissors_watermark_valign_bottom" type="radio" value="2" name="scissors_watermark_valign" <?php checked('2', get_option('scissors_watermark_valign')); ?> />
<label class="align_watermark" for="scissors_watermark_valign_bottom"><?php _e('bottom'); ?></label>
</fieldset><?php
}

function scissors_admin_head_watermark()
{
	global $scissors_dirname;
	if(strstr($_SERVER['REQUEST_URI'], 'options-media'))
	{
		$wpurl = get_bloginfo('wpurl');
		echo "<!-- JS loaded for Scissors in options-media -->\n";
		echo "<script type='text/javascript'>\n/* <![CDATA[ */\n";
		echo "scissors2 = {\n";
		echo "wpUrl: '$wpurl/'\n";
		echo "}\n";
		echo "/* ]]> */\n</script>\n";
		echo "<!-- End of JS loaded for Scissors in options-media -->\n";

		wp_enqueue_script('thickbox');
		wp_enqueue_script('scissors_watermark_select_js', '/' . PLUGINDIR . '/'.$scissors_dirname.'/js/watermark-select.js' );
	}
}

add_action('admin_print_scripts', 'scissors_admin_head_watermark');  // chaged to admin_print_scripts instead of admin_head for wp_enqueue_script to work properly

function scissors_whitelist_watermark_options($options)
{
	if(isset($options['media']))
	{
		$options['media'][] = 'scissors_watermark_enabled';
		$options['media'][] = 'scissors_watermark_path';
		$options['media'][] = 'scissors_watermark_size';
		$options['media'][] = 'scissors_watermark_size_relative';
		$options['media'][] = 'scissors_watermark_halign';
		$options['media'][] = 'scissors_watermark_valign';
	}
	return $options;
}

if(strstr($_SERVER['REQUEST_URI'], 'options'))
{
	add_option('scissors_watermark_enabled', '0');
	add_option('scissors_watermark_path', '');
	add_option('scissors_watermark_size', '1');
	add_option('scissors_watermark_size_relative', '20');
	add_option('scissors_watermark_halign', '2');
	add_option('scissors_watermark_valign', '2');

	add_filter('whitelist_options', 'scissors_whitelist_watermark_options');
	add_filter('sanitize_option_scissors_watermark_path', 'trim');
	add_filter('sanitize_option_scissors_watermark_size_relative', 'intval');

	$wp_settings_sections['media'][] = array('id' => 'watermarking', 'title' => __('Watermarking', 'scissors'), 'callback' => 'scissors_watermarking');
	$wp_settings_fields['media']['watermarking'][] = array('title' => __('Enable', 'scissors'), 'callback' => 'scissors_watermarking_switch');
	$wp_settings_fields['media']['watermarking'][] = array('title' => __('Image', 'scissors'), 'callback' => 'scissors_watermarking_path');
	$wp_settings_fields['media']['watermarking'][] = array('title' => __('Size', 'scissors'), 'callback' => 'scissors_watermarking_size');
	$wp_settings_fields['media']['watermarking'][] = array('title' => __('Horizontal alignment', 'scissors'), 'callback' => 'scissors_watermarking_halign');
	$wp_settings_fields['media']['watermarking'][] = array('title' => __('Vertical alignment', 'scissors'), 'callback' => 'scissors_watermarking_valign');
}

// Manual cropping and resizing functionality ---------------------------------

function scissors_admin_head()
{
	if(strstr($_SERVER['REQUEST_URI'], 'media'))
	{
		global $scissors_dirname;
		
		wp_enqueue_script('scissors_crop', '/' . PLUGINDIR . '/'.$scissors_dirname.'/js/jquery.Jcrop.js', array('jquery') );
		wp_enqueue_script('scissors_js', '/' . PLUGINDIR . '/'.$scissors_dirname.'/js/scissors.js' );
		
		$thisUrl = admin_url('admin-ajax.php');
		$thumbAspectRatio = intval(get_option("thumbnail_size_w")) / intval(get_option("thumbnail_size_h"));
		$cropThumbnails = (get_option("thumbnail_crop") == 1) ? 1 : 0;
		echo "<!-- JS loaded for Scissors in media library -->\n";
		echo "<script type='text/javascript'>\n/* <![CDATA[ */\n";
		echo "scissors = {\n";
		echo "ajaxUrl: '$thisUrl',\n";
		echo "thumbAspectRatio: $thumbAspectRatio,\n";
		echo "cropThumbnails: $cropThumbnails\n";
		echo "}\n";
		echo "/* ]]> */\n</script>\n";
		echo "<!-- End of JS loaded for Scissors in media library -->\n";
	}
}

function scissors_styles()
{
	global $scissors_dirname;

	$jcrop_style = '/' .PLUGINDIR . '/' . $scissors_dirname . '/css/jquery.Jcrop.css';
	$scissors_style = '/' .PLUGINDIR . '/' . $scissors_dirname . '/css/scissors.css';
	$thickbox_style = '/wp-includes/js/thickbox/thickbox.css';

	wp_register_style('jcrop_style', $jcrop_style);
	wp_enqueue_style( 'jcrop_style');

	wp_register_style('scissors_style', $scissors_style);
	wp_enqueue_style( 'scissors_style');

	wp_register_style('thickbox_style', $thickbox_style);
	wp_enqueue_style( 'thickbox_style');
}

function gcd($a, $b)
{
	if ($a == 0 || $b == 0)
		return 0;
	else if ($a == $b)
		return $a;
	else
	{
		do
		{
			$r = $a % $b;
			$a = $b; $b = $r;
		} while($r != 0);
		return $a;
	}
}

function scissors_media_meta($string, $post)
{
	if($post->post_mime_type == 'image/jpeg' || $post->post_mime_type == 'image/png' || $post->post_mime_type == 'image/gif')
	{
		$text_crop = __('Crop', 'scissors');
		$text_resize = __('Resize', 'scissors');
		$text_rotate = __('Rotate', 'scissors');
		$text_watermarks = __('Watermarks', 'scissors');
		$text_addwatermarks = __('Add watermarks', 'scissors');
		$text_removewatermarks = __('Remove watermarks', 'scissors');
		$text_abort = __('Abort', 'scissors');
		$text_crop2 = __('Crop %s.', 'scissors');
		$text_crop3 = __('Lock aspect ratio to %s.', 'scissors');
		$text_width = __('width', 'scissors');
		$text_height = __('height', 'scissors');
		$text_longside = __('long side', 'scissors');
		$text_shortside = __('short side', 'scissors');
		$text_resize2 = __('Resize %1$s to %2$s pixels.', 'scissors');
		$text_reir = __('Use automatic relevance-enhanced image reduction.', 'scissors');

		$image_url = wp_get_attachment_url($post->ID);
		$postId = $post->ID;
		$nonce = wp_create_nonce("scissors-$postId");

		$filename = get_attached_file($postId);
		if($filename && file_exists($filename))
			list($width, $height) = getimagesize($filename);
		$img_size = isset($width) ? ($width . '&nbsp;&times;&nbsp;' . $height) : '';
	
		$string .= "<span id='scissorsSize-$postId'>$img_size</span></td></tr>\n\t\t<tr>";
		$string .= "<th valign='top' scope='row' class='label'>Scissors</th><td class='field'>";
		$string .= "<div id='scissorsShowBtn-$postId'>";
		$string .= "<button class='button' onclick=\"return scissorsShowCrop($postId, '$image_url')\">$text_crop</button>&nbsp;";
		$string .= "<button class='button' onclick=\"return scissorsShowResize($postId)\">$text_resize</button>&nbsp;";
		$string .= "<button class='button' onclick=\"return scissorsShowRotate($postId)\">$text_rotate</button>&nbsp;";
		$string .= "<button class='button' onclick=\"return scissorsShowWatermarks($postId)\">$text_watermarks</button>";
		$string .= "</div>";

		$text_targets = array('chain' => __('full chain', 'scissors'), 'full' => __('full', 'scissors'),
			'large' => __('large', 'scissors'), 'medium' => __('medium', 'scissors'), 'thumbnail' => __('thumbnail', 'scissors'));
		$cropTargets = "<select id='scissorsCropTarget-$postId' onchange=\"scissorsCropTargetChange($postId)\">";
		$cropTargets .= "<option value='chain' selected='selected'>" . $text_targets['chain'] . "</option>";
		$cropTargets .= "<option value='full'>" . $text_targets['full'] . "</option>";
   		$sizes = apply_filters('intermediate_image_sizes', array('large', 'medium', 'thumbnail'));
		foreach($sizes as $size)
		{
			if($intermediate = image_get_intermediate_size($postId, $size))
			{
				$text_target = isset($text_targets[$size]) ? $text_targets[$size] : $size;
				$cropTargets .= "<option value='$size'>" . $text_target . "</option>";
			}
		}
		$cropTargets .= "</select>";

		$aspectX = isset($width) ? $width : 4;
		$aspectY = isset($height) ? $height : 3;
		while(($g = gcd($aspectX, $aspectY)) > 1)
		{
			$aspectX = intval($aspectX / $g);
			$aspectY = intval($aspectY / $g);
		}
		$cropAspect  = "<input style='width:2em;text-align:center;' type='text' onchange=\"scissorsAspectChange($postId)\" id='scissorsLockX-$postId' value='$aspectX' />:";
		$cropAspect .= "<input style='width:2em;text-align:center;' type='text' onchange=\"scissorsAspectChange($postId)\" id='scissorsLockY-$postId' value='$aspectY' />";

		$string .= "<div class='scissorsPane' id='scissorsCropPane-$postId'>";
		$string .= sprintf($text_crop2, $cropTargets);
		$string .= "&nbsp;<button class='button' onclick=\"return scissorsCrop($postId, '$nonce')\">$text_crop</button>";
		$string .= "&nbsp;<button class='button' onclick=\"return scissorsAbortCrop($postId)\">$text_abort</button>";
		$string .= "<div class='scissorsImgHost' id='scissorsImgHost-$postId'><img id='scissorsImg-$postId' /></div>";
		$string .= "<input type='hidden' id='scissorsX-$postId' /><input type='hidden' id='scissorsY-$postId' />";
		$string .= "<input type='hidden' id='scissorsW-$postId' /><input type='hidden' id='scissorsH-$postId' />";
		$string .= "<span id='scissorsSel-$postId'></span>";
		$string .= "<div id='scissorsAspect-$postId'><input type='checkbox' onchange=\"scissorsAspectChange($postId)\" id='scissorsLockBox-$postId' checked='checked' />&nbsp;";
		$string .= sprintf($text_crop3, $cropAspect);
		$string .= "</div><div id='scissorsReir-$postId'><input type='checkbox' id='scissorsReirEnable-$postId' />&nbsp;$text_reir&nbsp;<a href=\"http://www.useit.com/alertbox/9611.html\" target=\"_blank\">[?]</a></div>";
		$string .= "</div>";

		$resizeMode = "<select id='scissorsMode-$postId'><option value='width' selected='selected'>$text_width</option><option value='height'>$text_height</option><option value='long'>$text_longside</option><option value='short'>$text_shortside</option></select>";
		$resizeValue = "<input style='width:3em;text-align:center;' type='text' id='scissorsSide-$postId' value='640' />";
		$string .= "<div class='scissorsPane' id='scissorsResizePane-$postId'>";
		$string .= sprintf($text_resize2, $resizeMode, $resizeValue);
		$string .= "&nbsp;<button class='button' onclick=\"return scissorsResize($postId, '$nonce')\">$text_resize</button>";
		$string .= "&nbsp;<button class='button' onclick=\"return scissorsAbortResize($postId)\">$text_abort</button>";
		$string .= "</div>";

		$text_rotate2 = __('Rotate by', 'scissors');
		$text_left90 = __('90 degrees left', 'scissors');
		$text_right90 = __('90 degrees right', 'scissors');
		$text_180 = __('180 degrees', 'scissors');
		$string .= "<div class='scissorsPane' id='scissorsRotatePane-$postId'>";
		$string .= "$text_rotate2 <select id='scissorsRotateAngle-$postId'><option value='90' selected='selected'>$text_left90</option><option value='-90'>$text_right90</option><option value='180'>$text_180</option></select>.";
		$string .= "&nbsp;<button class='button' onclick=\"return scissorsRotate($postId, '$nonce')\">$text_rotate</button>";
		$string .= "&nbsp;<button class='button' onclick=\"return scissorsAbortRotate($postId)\">$text_abort</button>";
		$string .= "</div>";

		$string .= "<div class='scissorsPane' id='scissorsWatermarkPane-$postId'>";
		if(scissors_is_watermarking_enabled())
			$string .= "<button class='button' onclick=\"return scissorsWatermark($postId, '$nonce', true)\">$text_addwatermarks</button>&nbsp;";
		$string .= "<button class='button' onclick=\"return scissorsWatermark($postId, '$nonce', false)\">$text_removewatermarks</button>";
		$string .= "&nbsp;<button class='button' onclick=\"return scissorsAbortWatermark($postId)\">$text_abort</button>";
		$string .= "</div>";

		$string .= "<div class='scissorsPane scissorsWaitFld' id='scissorsWaitFld-$postId'></div>";
	}
	return $string;
}

function scissors_image_make_intermediate_size($file, $width, $height, $crop=false)
{
	$intermediate_result = image_make_intermediate_size($file, $width, $height, $crop);
	if(!$intermediate_result)
	{
		// simply use the specified file as the new intermediate albeit with smaller dimensions
		$suffix = "{$width}x{$height}";
		$info = pathinfo($file);
		$dir = $info['dirname'];
		$ext = $info['extension'];
		$name = basename($file, ".{$ext}");
		$destfilename = "{$dir}/{$name}-{$suffix}.{$ext}";
		copy($file, $destfilename);
		
		list($realW, $realH) = getimagesize($file);		
		$intermediate_result = array('file' => basename($destfilename), 'width' => $realW, 'height' => $realH);
	}
	return $intermediate_result;
}

function scissors_crop($post, $srcfile, $src)
{
	global $scissors_dirname;
	$x = intval($_POST['x']); $y = intval($_POST['y']);
	$w = intval($_POST['w']); $h = intval($_POST['h']);
	if($x >= 0 && $y >= 0 && $w > 0 && $h > 0)
	{
		$fullfile = get_attached_file($post->ID);
		if($_POST['target'] == 'chain' || $_POST['target'] == 'full')
			$dstfile = $fullfile;
		else if($intermediate = image_get_intermediate_size($post->ID, $_POST['target']))
			$dstfile = tempnam(dirname($fullfile), 'scissors');
		else
			return __('Invalid target.', 'scissors');

		$dst = scissors_create_image($w, $h);
		if(is_resource($dst))
		{
			if(imagecopy($dst, $src, 0, 0, $x, $y, $w, $h))
			{
				if(scissors_save_image($dst, $dstfile, $post->post_mime_type))
				{
					if($dstfile == $fullfile)
						scissors_rebuild_watermark_meta($dst, $post->ID);

					$metadata = wp_get_attachment_metadata($post->ID);
					if($_POST['target'] == 'chain' || $_POST['target'] == 'full')
					{
						// update meta data for the full-size image
						$metadata['width'] = $w; $metadata['height'] = $h;
						list($uwidth, $uheight) = wp_shrink_dimensions($metadata['width'], $metadata['height']);
						$metadata['hwstring_small'] = "height='$uheight' width='$uwidth'";

						$msg = "done;full,{$w},{$h}"; // signal change of the full-size image

						// update existing intermediate images if requested
						if($_POST['target'] == 'chain' && count($metadata['sizes']) > 0)
						{
							foreach($metadata['sizes'] as $size => $data)
							{
								$srcfile = $fullfile;
								$w2 = get_option("{$size}_size_w");
								$h2 = get_option("{$size}_size_h");
								if($_POST['reir'] == '1')
								{
									$fx = $w2 / $w; $fy = $h2 / $h;
									$factor = sqrt($fx > $fy ? $fx : $fy);
									$cw = ceil($w * $factor); $ch = ceil($h * $factor);
									if($cw != $w || $ch != $h)
									{
										$cx = floor(($w - $cw) / 2); $cy = floor(($h - $ch) / 2);
										$interfile = tempnam(dirname($fullfile), 'scissors');

										// crop $fullfile to ($cx,$cy) [$cw,$ch] and store to $interfile
										$tmp = scissors_create_image($cw, $ch);
										if(is_resource($tmp))
										{
											if(imagecopy($tmp, $dst, 0, 0, $cx, $cy, $cw, $ch))
											{
												if(scissors_save_image($tmp, $interfile, $post->post_mime_type))
													$srcfile = $interfile;
											}
											imagedestroy($tmp);
										}

										if($srcfile != $interfile)
											unlink($interfile);
									}
								}
	
								$resized = scissors_image_make_intermediate_size($srcfile, $w2, $h2, get_option("{$size}_crop"));
								if($resized)
								{
									$oldfile = path_join(dirname($fullfile), $data['file']);

									if($resized['file'] != $data['file'])
									{
										// overwrite the old file with the newly created image
										$resizedfile = path_join(dirname($srcfile), $resized['file']);
										copy($resizedfile, $oldfile);
										unlink($resizedfile);
										$resized['file'] = $data['file'];
									}
									$metadata['sizes'][$size] = $resized;
									$msg .= "+$size," . $resized['width'] . ',' . $resized['height']; // signal change

									scissors_watermark_file($oldfile, $post->ID);
								}

								if($srcfile != $fullfile)
									unlink($srcfile); // clean up reir temporary
							}
						}

						wp_update_attachment_metadata($post->ID, $metadata);

						scissors_watermark_file($dstfile, $post->ID);
					}
					else
					{
						$size = $_POST['target'];
						$resized = scissors_image_make_intermediate_size($dstfile, get_option("{$size}_size_w"), get_option("{$size}_size_h"), get_option("{$size}_crop"));
						if($resized)
						{
							// overwrite the specified destination
							$resizedfile = path_join(dirname($dstfile), $resized['file']);
							$oldfile = path_join(dirname($fullfile), $intermediate['file']);
							if(copy($resizedfile, $oldfile))
							{
								$resized['file'] = $intermediate['file'];
								$metadata['sizes'][$size] = $resized;
								$msg = "done;$size," . $resized['width'] . ',' . $resized['height']; // signal change
								scissors_watermark_file($oldfile, $post->ID);
							}
							else
								$msg = __('Failed to save new image.', 'scissors');

							unlink($resizedfile);
						}
						else
							$msg = __('Failed to save new image.', 'scissors');

						wp_update_attachment_metadata($post->ID, $metadata);
					}
				}
				else
					$msg = __('Failed to save new image.', 'scissors');
			}
			else
				$msg = __('Failed to crop image.', 'scissors');
	
			imagedestroy($dst);
		}
		else
			$msg = __('Failed to allocate memory.', 'scissors');

		// cleanup temporary file
		if($intermediate)
			unlink($dstfile);
		return $msg;
	}
	else
		return __('Invalid selection.', 'scissors');
}

function scissors_resize($post, $filename, $src)
{
	$side = intval($_POST['side']);
	if($side > 0)
	{
		$srcW = imagesx($src); $srcH = imagesy($src);
		switch($_POST['mode'])
		{
		case 'width': $w = $side; $h = ceil($srcH * $side / $srcW); break;
		case 'height': $h = $side; $w = ceil($srcW * $side / $srcH); break;
		case 'long':
			$w = ($srcW >= $srcH) ? $side : ceil($srcW * $side / $srcH);
			$h = ($srcH > $srcW) ? $side : ceil($srcH * $side / $srcW);
			break;
		case 'short':
			$w = ($srcW < $srcH) ? $side : ceil($srcW * $side / $srcH);
			$h = ($srcH <= $srcW) ? $side : ceil($srcH * $side / $srcW);
			break;
		default:
			return __('Invalid mode.', 'scissors');
		}
	
		if($w > 0 && $h > 0)
		{
			if($w != $srcW || $h != $srcH)
			{
				$dst = scissors_create_image($w, $h);
				if(is_resource($dst))
				{
					if(imagecopyresampled($dst, $src, 0, 0, 0, 0, $w, $h, $srcW, $srcH))
					{
						scissors_rebuild_watermark_meta($dst, $post->ID);
						scissors_watermark_image($dst, $post->ID);

						if(scissors_save_image($dst, $filename, $post->post_mime_type))
						{
							// update meta data, no need to rebuild intermediate images because the aspect ratio always stays the same
							$metadata = wp_get_attachment_metadata($post->ID);
							$metadata['width'] = $w; $metadata['height'] = $h;
							list($uwidth, $uheight) = wp_shrink_dimensions($metadata['width'], $metadata['height']);
							$metadata['hwstring_small'] = "height='$uheight' width='$uwidth'";
							wp_update_attachment_metadata($post->ID, $metadata);
							$msg = "done;full,{$w},{$h}"; // signal change of the full-size image
						}
						else
							$msg = __('Failed to save new image.', 'scissors');
					}
					else
						$msg = __('Failed to resize image.', 'scissors');

					imagedestroy($dst);
					return $msg;
				}
				else
					return __('Failed to allocate memory.', 'scissors');
			}
			else
				return 'done;*'; // signal no change
		}
		else
			return __('Invalid destination dimensions.', 'scissors');
	}
	else
		return __('Invalid side length.', 'scissors');
}

function scissors_per_image_watermarks($post, $srcfile, $src)
{
	switch($_POST['state'])
	{
	case 'true': scissors_set_watermarking_state($post->ID, true); break;
	case 'false': scissors_set_watermarking_state($post->ID, false); break;
	default: return __('Invalid query.', 'scissors');
	}

	// reuse the cropping functionality to rebuild the image chain with/without watermarking
	$_POST['x'] = 0; $_POST['y'] = 0;
	$_POST['w'] = imagesx($src); $_POST['h'] = imagesy($src);
	$_POST['target'] = 'chain'; $_POST['reir'] = 0;
	return scissors_crop($post, $srcfile, $src);
}

function scissors_rotate($post, $srcfile, $src)
{
	$angle = intval($_POST['angle']);
	switch($angle)
	{
	case -90: case 90: case 180: $rotated = imagerotate($src, $angle, 0); break;
	default: return __('Invalid query.', 'scissors');
	}

	// reuse the cropping functionality to rebuild the image chain after rotation
	$_POST['x'] = 0; $_POST['y'] = 0;
	$_POST['w'] = imagesx($rotated); $_POST['h'] = imagesy($rotated);
	$_POST['target'] = 'chain'; $_POST['reir'] = 0;
	return scissors_crop($post, $srcfile, $rotated);
}

function scissors_action()
{
	$postId = intval($_POST['post_id']); // perform security checks
	if (wp_verify_nonce($_POST['_wpnonce'], "scissors-$postId") && current_user_can('upload_files'))
	{
		// retrieve post data and load the full-size image
		$post = get_post($postId);
		$filename = get_attached_file($postId);
		if($post && $filename && file_exists($filename))
		{
			scissors_set_memory_limit();
			$src = scissors_load_image($filename, $post->post_mime_type); // load the source image			
			if(is_resource($src))
			{
				// if the full image contains a watermark we have to remove it first ...
				scissors_remove_watermark($src, $postId);

				if($_POST['action'] == 'scissorsCrop')
					$msg = scissors_crop($post, $filename, $src);
				else if($_POST['action'] == 'scissorsResize')
					$msg = scissors_resize($post, $filename, $src);
				else if($_POST['action'] == 'scissorsRotate')
					$msg = scissors_rotate($post, $filename, $src);
				else if($_POST['action'] == 'scissorsWatermark')
					$msg = scissors_per_image_watermarks($post, $filename, $src);
				else
					$msg = __('Invalid query.', 'scissors');

				imagedestroy($src);
			}
			else
				$msg = __('Failed to load image.', 'scissors');
		}
		else
			$msg = __('Invalid post-id.', 'scissors');
	}
	else
		$msg = __('Unauthorized query.', 'scissors');
	die($msg);
}

add_filter('media_meta', 'scissors_media_meta', 99, 2);
add_action('admin_print_scripts', 'scissors_admin_head');
add_action('admin_print_styles', 'scissors_styles');
add_action('wp_ajax_scissorsCrop', 'scissors_action');
add_action('wp_ajax_scissorsResize', 'scissors_action');
add_action('wp_ajax_scissorsRotate', 'scissors_action');
add_action('wp_ajax_scissorsWatermark', 'scissors_action');

// On-the-fly image resampling functionality ----------------------------------

class Scissors
{
	function Scissors()
	{
		add_filter('mce_external_plugins', array(&$this, 'mce_external_plugins'));
		add_filter('content_save_pre', array(&$this, 'content_save_pre'));
		add_action('save_post', array(&$this, 'save_post'));
		add_action('delete_post', array(&$this, 'delete_post'));

		// NOTE: cannot use the delete_attachment-action because at the time it is invoked the metadata
		// associated with the attachment is already gone. wow, what a stupid way to implement this!
		add_filter('wp_delete_file', array(&$this, 'wp_delete_file'));	
	}

	function mce_external_plugins($plugins)
	{
		$plugins['scissors'] = plugins_url('/scissors/mce/editor_plugin.js');
		return $plugins;
	}

	function _getImagePostIdFromUrl($url)
	{
		$wud = wp_upload_dir(); // remove the base url
		$url = str_replace($wud['baseurl'] . '/', '', $url);

		// remove -(width)x(height) and -custom
		if(preg_match('!(-[0-9]+x[0-9]+(-custom)?)([^/\\\]+)$!', $url, $match) == 1)
			$url = str_replace($match[1], '', $url);

		global $wpdb;
		$post = $wpdb->get_row($wpdb->prepare("SELECT post_id FROM $wpdb->postmeta WHERE meta_value = %s LIMIT 1", $url));
		return ($post != null) ? $post->post_id : FALSE;
	}

	function _createResampledImage($imgurl, $destWidth, $destHeight)
	{
		$retval = FALSE;
		$postId = $this->_getImagePostIdFromUrl($imgurl);
		if($postId != FALSE)
		{
			// retrieve post data and load the full-size image
			$post = get_post($postId);
			$filename = get_attached_file($postId);
			if($post && $filename && file_exists($filename))
			{
				$url = wp_get_attachment_url($postId);
				list($fullWidth, $fullHeight) = getimagesize($filename);
				if($destWidth < $fullWidth || $destHeight < $fullHeight)
				{
					$path_parts = pathinfo($filename);
					$sansext = substr($path_parts['basename'], 0, -(strlen($path_parts['extension']) + 1)); // +1 for the dot
					$path_parts['basename'] = "{$sansext}-{$destWidth}x{$destHeight}-custom." . $path_parts['extension'];
					$url = str_replace(basename($url), $path_parts['basename'], $url);							
					$destfilename = $path_parts["dirname"] . '/' . $path_parts['basename'];
					if(!file_exists($destfilename))
					{
						scissors_set_memory_limit();
						$src = scissors_load_image($filename, $post->post_mime_type); // load the source image
						if(is_resource($src))
						{
							$dst = scissors_create_image($destWidth, $destHeight);
							if(is_resource($dst))
							{
								// if the full image contains a watermark we have to remove it first ...
								scissors_remove_watermark($src, $postId);

								if(imagecopyresampled($dst, $src, 0, 0, 0, 0, $destWidth, $destHeight, imagesx($src), imagesy($src)))
								{
									// add the watermark to the resampled image
									scissors_watermark_image($dst, $postId);

									if(scissors_save_image($dst, $destfilename, $post->post_mime_type))
									{
										$key = basename($url);

										add_post_meta($postId, "_scissors_custom_images", array(), true); // initialize if non-existant
										$meta = get_post_meta($postId, '_scissors_custom_images', true);
										$meta[$key] = array('ref' => '');
										update_post_meta($postId, "_scissors_custom_images", $meta);

										$retval = $url;
									}
								}
								imagedestroy($dst);
							}
							imagedestroy($src);
						}
					}
					else
						$retval = $url;
				}
				else
					$retval = $url;
			}
		}
		return $retval;
	}

	function content_save_pre($content)
	{
		// find images with class 'scissors-resample' and resample the image to the specified size
		$pattern = '!(<)img[^>]+class=\\\"[^">]*scissors-resample[^">]*\\\"[^>]*(>)!is';
		$count = preg_match_all($pattern, $content, $matches, PREG_OFFSET_CAPTURE | PREG_PATTERN_ORDER);
		if($count > 0)
		{
			$newcontent = ''; $last = 0;
			for($i = 0; $i < $count; $i++)
			{
				$tagStart = $matches[1][$i][1]; $tagEnd = $matches[2][$i][1];
				$newcontent .= substr($content, $last, $tagStart - $last); // append the content up to the tag
				$last = $tagEnd + 1; // skip over the tag itself

				$tag = substr($content, $tagStart, $tagEnd - $tagStart + 1);				
				$tag = str_replace('scissors-resample', '', str_replace(' scissors-resample', '', $tag)); // remove the class

				$width = -1; $height = -1; $url = '';
				if(preg_match('!width=\\\"([0-9]+)\\\"!is', $tag, $widthmatch) == 1)
					$width = ($widthmatch[1]);
				if(preg_match('!height=\\\"([0-9]+)\\\"!is', $tag, $heightmatch) == 1)
					$height = ($heightmatch[1]);
				if(preg_match('!src=\\\"([^">]*)\\\"!is', $tag, $urlmatch, PREG_OFFSET_CAPTURE) == 1)
					$url = $urlmatch[1][0];

				if($width > 0 && $height > 0 && $url != '')
				{
					$newurl = $this->_createResampledImage($url, $width, $height);
					if($newurl != FALSE)
						$tag = str_replace($url, $newurl, $tag);
				}

				$newcontent .= $tag; // append the modified tag
			}
			$newcontent .= substr($content, $last);
			return $newcontent;
		}
		else
			return $content;
	}

	function _deleteReferenceToCustomImage($imgPostId, $key, $postId)
	{
		$meta = get_post_meta($imgPostId, '_scissors_custom_images', true);
		if($meta != FALSE)
		{
			if(strstr($meta[$key]['ref'], "[$postId]") != FALSE)
			{
				$meta[$key]['ref'] = str_replace("[$postId]", '', $meta[$key]['ref']);
				if($meta[$key]['ref'] == '')
				{
					$filename = get_attached_file($imgPostId);
					$filename = str_replace(basename($filename), $key, $filename);
					@unlink($filename);						
					unset($meta[$key]);
				}
	
				if(count($meta) > 0)
					update_post_meta($imgPostId, "_scissors_custom_images", $meta);
				else
					delete_post_meta($imgPostId, "_scissors_custom_images");
			}
		}
	}

	function save_post($postId)
	{
		$post = get_post($postId);

		$newimglist = array();

		// find references to custom images
		$pattern = '!<img[^>]+src="([^">]+-[0-9]+x[0-9]+-custom[^">/\\\]+)"[^>]*>!is';
		$count = preg_match_all($pattern, $post->post_content, $matches, PREG_PATTERN_ORDER);
		foreach($matches[1] as $url)
		{
			$imgPostId = $this->_getImagePostIdFromUrl($url);
			if($imgPostId != FALSE)
			{
				$meta = get_post_meta($imgPostId, '_scissors_custom_images', true);
				if($meta != FALSE)
				{
					$key = basename($url);
					$newimglist[] = "$imgPostId:$key";
					if(strstr($meta[$key]['ref'], "[$postId]") == FALSE)
					{
						$meta[$key]['ref'] .= "[$postId]";
						update_post_meta($imgPostId, "_scissors_custom_images", $meta);
					}
				}
			}
		}

		$newimglist = array_unique($newimglist);

		// if this post gets edited, make sure to remove references to images
		// that are no longer present in the new version
		$oldimglist = get_post_meta($postId, '_scissors_custom_images', true);
		if(is_array($oldimglist))
		{
			$oldimglist = array_diff($oldimglist, $newimglist);
			foreach($oldimglist as $elem)
			{
				list($imgPostId, $key) = explode(':', $elem);
				$this->_deleteReferenceToCustomImage($imgPostId, $key, $postId);
			}
		}

		if(count($newimglist) > 0)
		{
			add_post_meta($postId, "_scissors_custom_images", $newimglist, true) or
				update_post_meta($postId, "_scissors_custom_images", $newimglist);
		}
		else
			delete_post_meta($postId, "_scissors_custom_images");
	}

	function delete_post($postId)
	{
		$post = get_post($postId);

		// find references to custom images
		$pattern = '!<img[^>]+src="([^">]+-[0-9]+x[0-9]+-custom[^">/\\\]+)"[^>]*>!is';
		$count = preg_match_all($pattern, $post->post_content, $matches, PREG_PATTERN_ORDER);
		foreach($matches[1] as $url)
		{
			$imgPostId = $this->_getImagePostIdFromUrl($url);
			if($imgPostId != FALSE)
			{
				$key = basename($url);
				$this->_deleteReferenceToCustomImage($imgPostId, $key, $postId);
			}
		}
	}

	function wp_delete_file($filename)
	{
		if(file_exists($filename))
		{
			$path_parts = pathinfo($filename);
			$sansext = substr($path_parts['basename'], 0, -(strlen($path_parts['extension']) + 1)); // +1 for the dot
			$pattern = '!' . preg_quote($sansext) . '-[0-9]+x[0-9]+-custom' . preg_quote('.' . $path_parts['extension']) . '$!';

			$dir = opendir($path_parts['dirname']);
			while($file = readdir($dir))
			{
				if(preg_match($pattern, $file) == 1)
					@unlink($path_parts['dirname'] . '/' . $file);						
			}
			closedir($dir);
		}
		return $filename;
	}
}

$ScissorsInstance = new Scissors();

?>
